@@ -12,7 +12,7 @@ extern "C" {
 #include SWSCALE_INCLUDE
 }
 
-#define USE_FFMPEG_CRASH_RECOVERY
+// #define USE_FFMPEG_CRASH_RECOVERY
 #define AUDIO_CHANNELS 2
 
 /*static*/ boost::mutex FFmpeg::s_avcodec_mutex;
@@ -27,7 +27,7 @@ FFmpeg::FFmpeg(bool _decodeVideo, bool _decodeAudio, std::string const& _filenam
 FFmpeg::~FFmpeg() {
 	m_quit = true;
 	videoQueue.reset();
-	audioQueue.reset();
+	audioQueue.quit();
 	if (!m_thread) {
 		std::cerr << "FFMPEG crashed at some point, decoding " << m_filename << std::endl;
 		std::cerr << "Please restart Performous to avoid resource leaks and program crashes!" << std::endl;
@@ -141,18 +141,19 @@ void FFmpeg::operator()() {
 	sigabrt = std::signal(SIGABRT, performous_ffmpeg_crash_hack);
 	sigsegv = std::signal(SIGSEGV, performous_ffmpeg_crash_hack);
 #endif
-	try { open(); } catch (std::exception const& e) { std::cerr << "FFMPEG failed to open " << m_filename << ": " << e.what() << std::endl; m_quit = true; }
+	try { open(); } catch (std::exception const& e) { std::cerr << "FFMPEG failed to open " << m_filename << ": " << e.what() << std::endl; m_quit = true; return; }
 	m_running = true;
+	audioQueue.setDuration(duration());
 	int errors = 0;
 	while (!m_quit) {
 		try {
+			if (audioQueue.wantSeek()) m_seekTarget = 0.0;
 			if (m_seekTarget == m_seekTarget) seek_internal();
 			decodeNextFrame();
 			m_eof = false;
 			errors = 0;
 		} catch (eof_error&) {
 			m_eof = true;
-			audioQueue.push(new AudioFrame()); // EOF marker
 			videoQueue.push(new VideoFrame()); // EOF marker
 			boost::thread::sleep(now() + 0.1);
 		} catch (std::exception& e) {
@@ -162,7 +163,6 @@ void FFmpeg::operator()() {
 	}
 	m_running = false;
 	m_eof = true;
-	audioQueue.push(new AudioFrame()); // EOF marker
 	videoQueue.push(new VideoFrame()); // EOF marker
 }
 
@@ -173,8 +173,8 @@ void FFmpeg::seek(double time, bool wait) {
 }
 
 void FFmpeg::seek_internal() {
-	audioQueue.reset();
 	videoQueue.reset();
+	audioQueue.reset();
 	int flags = 0;
 	if (m_seekTarget < position()) flags |= AVSEEK_FLAG_BACKWARD;
 	int stream = -1;
@@ -231,7 +231,11 @@ void FFmpeg::decodeNextFrame() {
 			while (packetSize) {
 				if (packetSize < 0) throw std::logic_error("negative video packet size?!");
 				if (m_quit || m_seekTarget == m_seekTarget) return;
+#if LIBAVCODEC_VERSION_INT > ((52<<16)+(25<<8)+0)
+				int decodeSize = avcodec_decode_video2(pVideoCodecCtx, videoFrame, &frameFinished, &packet);
+#else
 				int decodeSize = avcodec_decode_video(pVideoCodecCtx, videoFrame, &frameFinished, packetData, packetSize);
+#endif
 				if (decodeSize < 0) throw std::runtime_error("cannot decode video frame");
 				// Move forward within the packet
 				packetSize -= decodeSize;
@@ -247,9 +251,10 @@ void FFmpeg::decodeNextFrame() {
 						sws_scale(img_convert_ctx, videoFrame->data, videoFrame->linesize, 0, h, &data, &linesize);
 					}
 					if (packet.time() == packet.time()) m_position = packet.time();
+					// Construct a new video frame and push it to output queue
 					VideoFrame* tmp = new VideoFrame(m_position, w, h);
 					tmp->data.swap(buffer);
-					videoQueue.push(tmp);
+					videoQueue.push(tmp); // Takes ownership and may block
 				}
 			}
 		} else if (decodeAudio && packet.stream_index==audioStream) {
@@ -258,7 +263,11 @@ void FFmpeg::decodeNextFrame() {
 				if (m_quit || m_seekTarget == m_seekTarget) return;
 				AudioBuffer audioFrames(AVCODEC_MAX_AUDIO_FRAME_SIZE);
 				int outsize = AVCODEC_MAX_AUDIO_FRAME_SIZE*sizeof(int16_t);
+#if LIBAVCODEC_VERSION_INT > ((52<<16)+(25<<8)+0)
+				int decodeSize = avcodec_decode_audio3(pAudioCodecCtx, audioFrames, &outsize, &packet);
+#else
 				int decodeSize = avcodec_decode_audio2(pAudioCodecCtx, audioFrames, &outsize, packetData, packetSize);
+#endif
 				// Handle special cases
 				if (decodeSize == 0) break;
 				if (outsize == 0) continue;
@@ -270,13 +279,12 @@ void FFmpeg::decodeNextFrame() {
 				outsize /= sizeof(int16_t) * pAudioCodecCtx->channels;
 				std::vector<int16_t> resampled(AVCODEC_MAX_AUDIO_FRAME_SIZE);
 				int frames = audio_resample(pResampleCtx, &resampled[0], audioFrames, outsize);
-				// Construct AudioFrame and add it to the queue
-				AudioFrame* tmp = new AudioFrame();
-				std::copy(resampled.begin(), resampled.begin() + frames * AUDIO_CHANNELS, std::back_inserter(tmp->data));
+				resampled.resize(frames * AUDIO_CHANNELS);
+				// Calculate new positions
 				if (packet.time() == packet.time()) m_position = packet.time();
-				else m_position += double(tmp->data.size())/double(audioQueue.getSamplesPerSecond());
-				tmp->timestamp = m_position;
-				audioQueue.push(tmp);
+				else m_position += double(resampled.size())/double(audioQueue.getSamplesPerSecond());
+				// Push to output queue (may block)
+				audioQueue.push(resampled, m_position);
 			}
 			// Audio frames are always finished
 			frameFinished = 1;
