@@ -1,27 +1,39 @@
 #include "config.hh"
 #include "fs.hh"
+#include "record.hh"
 #include "screen.hh"
+#include "joystick.hh"
+#include "songs.hh"
+#include "backgrounds.hh"
+#include "database.hh"
+#include "xtime.hh"
+#include "video_driver.hh"
+#include "i18n.hh"
+
+// Screens
 #include "screen_intro.hh"
 #include "screen_songs.hh"
 #include "screen_sing.hh"
 #include "screen_practice.hh"
 #include "screen_configuration.hh"
-#include "video_driver.hh"
-#include "xtime.hh"
+#include "screen_players.hh"
+#include "screen_hiscore.hh"
+
 #include <boost/format.hpp>
 #include <boost/program_options.hpp>
-// Needs at least Boost 1.36 and many systems don't have that: #include <boost/spirit/include/classic_core.hpp>
-#include <boost/spirit/core.hpp>
+#include <boost/spirit/include/classic_core.hpp>
 #include <boost/thread.hpp>
 #include <libda/audio.hpp>
 #include <csignal>
 #include <fstream>
-#include <set>
 #include <string>
 #include <vector>
+#include <cstdlib>
 
 volatile bool g_quit = false;
 
+bool g_take_screenshot = false;
+
 extern "C" void quit(int) {
 	g_quit = true;
 }
@@ -34,7 +46,6 @@ static void checkEvents_SDL(ScreenManager& sm, Window& window) {
 		std::cout << "Terminating, please wait... (or kill the process)" << std::endl;
 		throw QuitNow();
 	}
-	static bool esc = false;
 	SDL_Event event;
 	while(SDL_PollEvent(&event) == 1) {
 		switch(event.type) {
@@ -44,27 +55,26 @@ static void checkEvents_SDL(ScreenManager& sm, Window& window) {
 		  case SDL_VIDEORESIZE:
 			window.resize(event.resize.w, event.resize.h);
 			break;
-		  case SDL_KEYUP:
-			if (event.key.keysym.sym == SDLK_ESCAPE) esc = false;
-			break;
 		  case SDL_KEYDOWN:
 			int keypressed  = event.key.keysym.sym;
 			SDLMod modifier = event.key.keysym.mod;
-			// Workaround for key repeat on escape
-			if (keypressed == SDLK_ESCAPE) {
-				if (esc) return;
-				esc = true;
-			}
-			if (keypressed == SDLK_RETURN && modifier & KMOD_ALT ) {
+			if ((keypressed == SDLK_RETURN && modifier & KMOD_ALT) || keypressed == SDLK_F11) {
 				config["graphic/fullscreen"].b() = !config["graphic/fullscreen"].b();
 				continue; // Already handled here...
 			}
+			if (keypressed == SDLK_PRINT || keypressed == SDLK_F12) {
+				g_take_screenshot = true;
+				continue;
+			}
 			if (keypressed == SDLK_F4 && modifier & KMOD_ALT) {
 				sm.finished();
 				continue; // Already handled here...
 			}
 			break;
 		}
+		// Forward to screen even if the input system takes it (ignoring pushEvent return value)
+		// This is needed to allow navigation (quiting the song) to function even then
+		input::SDL::pushEvent(event);
 		sm.getCurrentScreen()->manageEvent(event);
 		switch(glGetError()) {
 			case GL_INVALID_ENUM: std::cerr << "OpenGL error: invalid enum" << std::endl; break;
@@ -81,7 +91,7 @@ static void checkEvents_SDL(ScreenManager& sm, Window& window) {
 
 void audioSetup(Capture& capture, Audio& audio) {
 	// initialize audio argument parser
-	using namespace boost::spirit; //::classic;
+	using namespace boost::spirit::classic;
 	unsigned channels, rate, frames;
 	std::string devstr;
 	// channel       ::= "channel=" integer
@@ -124,40 +134,56 @@ void audioSetup(Capture& capture, Audio& audio) {
 	if (!audio.isOpen()) std::cerr << "No playback devices could be used. Please use --pdev to define one." << std::endl;
 }
 
-std::string songlist;
-
-void mainLoop() {
+void mainLoop(std::string const& songlist) {
 	try {
 		Capture capture;
 		Audio audio;
 		audioSetup(capture, audio);
-		Songs songs(songlist);
+		Backgrounds backgrounds;
+		Database database(getConfigDir() / "database.xml");
+		Songs songs(database, songlist);
 		ScreenManager sm;
-		Window window(config["graphic/window_width"].i(), config["graphic/window_height"].i(), config["graphic/fullscreen"].b(), config["graphic/fs_width"].i(), config["graphic/fs_height"].i());
+		Window window(config["graphic/window_width"].i(), config["graphic/window_height"].i(), config["graphic/fullscreen"].b());
 		sm.addScreen(new ScreenIntro("Intro", audio, capture));
-		sm.addScreen(new ScreenSongs("Songs", audio, songs));
-		sm.addScreen(new ScreenSing("Sing", audio, songs, capture));
+		sm.addScreen(new ScreenSongs("Songs", audio, songs, database));
+		sm.addScreen(new ScreenSing("Sing", audio, capture, database, backgrounds));
 		sm.addScreen(new ScreenPractice("Practice", audio, capture));
 		sm.addScreen(new ScreenConfiguration("Configuration", audio));
+		sm.addScreen(new ScreenPlayers("Players", audio, database));
+		sm.addScreen(new ScreenHiscore("Hiscore", audio, songs, database));
 		sm.activateScreen("Intro");
 		// Main loop
 		boost::xtime time = now();
-		unsigned nb_frames = 0;
+		unsigned frames = 0;
 		while (!sm.isFinished()) {
-			checkEvents_SDL(sm, window);
-			window.blank();
-			sm.getCurrentScreen()->draw();
-			window.swap();
-			++nb_frames;
-			if (config["graphic/fps"].b()) {
-				if (now() - time > 1.0) {
-					std::cout << nb_frames << " FPS" << std::endl;
-					time += 1.0;
-					nb_frames = 0;
+			if( g_take_screenshot ) {
+				fs::path filename;
+				try {
+					window.screenshot();
+				} catch (std::exception& e) {
+					std::cerr << "ERROR: " << e.what() << std::endl;
 				}
-			} else {
-				boost::thread::sleep(time + 0.01); // Max 100 FPS
-				time = now();
+				g_take_screenshot = false;
+			}
+			try {
+				checkEvents_SDL(sm, window);
+				window.blank();
+				sm.getCurrentScreen()->draw();
+				window.swap();
+				if (config["graphic/fps"].b()) {
+					++frames;
+					if (now() - time > 1.0) {
+						std::cout << frames << " FPS" << std::endl;
+						time += 1.0;
+						frames = 0;
+					}
+				} else {
+					boost::thread::sleep(time + 0.01); // Max 100 FPS
+					time = now();
+					frames = 0;
+				}
+			} catch (std::runtime_error& e) {
+				std::cerr << "ERROR: " << e.what() << std::endl;
 			}
 		}
 	} catch (std::exception& e) {
@@ -167,6 +193,37 @@ void mainLoop() {
 	}
 }
 
+/// Simple test utility to make mapping of joystick buttons/axes easier
+void jstestLoop() {
+	try {
+		Window window(config["graphic/window_width"].i(), config["graphic/window_height"].i(), false);
+		// Main loop
+		boost::xtime time = now();
+		while (true) {
+			SDL_Event e;
+			while(SDL_PollEvent(&e) == 1) {
+				if (e.type == SDL_QUIT || (e.type == SDL_KEYDOWN && e.key.keysym.sym == SDLK_ESCAPE)) {
+					return;
+				} else if (e.type == SDL_JOYBUTTONDOWN) {
+					std::cout << "JoyID: " << int(e.jbutton.which) << ", button: " << int(e.jbutton.button) << ", state: " << int(e.jbutton.state) << std::endl;
+				} else if (e.type == SDL_JOYAXISMOTION) {
+					std::cout << "JoyID: " << int(e.jaxis.which) << ", axis: " << int(e.jaxis.axis) << ", value: " << int(e.jaxis.value) << std::endl;
+				} else if (e.type == SDL_JOYHATMOTION) {
+					std::cout << "JoyID: " << int(e.jhat.which) << ", hat: " << int(e.jhat.hat) << ", value: " << int(e.jhat.value) << std::endl;
+				}
+			}
+			window.blank(); window.swap();
+			boost::thread::sleep(time + 0.01); // Max 100 FPS
+			time = now();
+		}
+	} catch (std::exception& e) {
+		std::cout << "ERROR: " << e.what() << std::endl;
+	} catch (QuitNow&) {
+		std::cout << "Terminated." << std::endl;
+	}
+	return;
+}
+
 template <typename Container> void confOverride(Container const& c, std::string const& name) {
 	if (c.empty()) return;  // Don't override if no options specified
 	ConfigItem::StringList& sl = config[name].sl();
@@ -175,50 +232,55 @@ template <typename Container> void confOverride(Container const& c, std::string
 }
 
 int main(int argc, char** argv) {
+#ifdef USE_GETTEXT
+	// initialize gettext
+#ifdef _MSC_VER
+	setlocale(LC_ALL, "");//only untill we don't have a better solution. This because LC_MESSAGES cause crash under Visual Studio
+#else
+	setlocale (LC_MESSAGES, "");
+#endif
+	bindtextdomain (PACKAGE, LOCALEDIR);
+	textdomain (PACKAGE);
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
+#endif
+
+	std::cout << PACKAGE " " VERSION << std::endl;
 	std::signal(SIGINT, quit);
 	std::signal(SIGTERM, quit);
 	std::ios::sync_with_stdio(false);  // We do not use C stdio
+	std::srand(std::time(NULL));
 	// Parse commandline options
 	std::vector<std::string> mics;
 	std::vector<std::string> pdevs;
 	std::vector<std::string> songdirs;
 	namespace po = boost::program_options;
 	po::options_description opt1("Generic options");
+	std::string songlist;
 	opt1.add_options()
 	  ("help,h", "you are viewing it")
-	  ("version,v", "display version number");
+	  ("version,v", "display version number")
+	  ("songlist", po::value<std::string>(&songlist), "save a list of songs in the specified folder");
 	po::options_description opt2("Configuration options");
 	opt2.add_options()
-	  ("theme,t", po::value<std::string>(), "set theme (name or absolute path)")
-	  ("fs,f", "enable full screen mode")
-	  ("fps", "benchmark rendering speed\n  also disable 100 FPS limit")
-	  ("songlist", po::value<std::string>(&songlist), "save a list of songs in the specified folder")
-	  ("width,W", po::value<int>(), "set horizontal resolution")
-	  ("height,H", po::value<int>(), "set vertical resolution")
-	  ("fs-width", po::value<int>(), "set fullscreen horizontal resolution")
-	  ("fs-height", po::value<int>(), "set fullscreen vertical resolution")
-	  ("michelp", "detailed help for --mics and a list of available audio devices")
-	  ("mics", po::value<std::vector<std::string> >(&mics)->composing(), "specify microphones to use")
-	  ("pdevhelp", "detailed help for --pdev and a list of available audio devices")
-	  ("pdev", po::value<std::vector<std::string> >(&pdevs)->composing(), "specify a playback device")
-	  ("songdir,s", po::value<std::vector<std::string> >(&songdirs)->composing(), "additional song folders to scan\n  may be specified without -s or -songdir too");
+	  ("mics", po::value<std::vector<std::string> >(&mics)->composing(), "specify the microphones to use")
+	  ("pdev", po::value<std::vector<std::string> >(&pdevs)->composing(), "specify the playback device")
+	  ("michelp", "detailed help and device list for --mics")
+	  ("pdevhelp", "detailed help and device list for --pdev")
+	  ("jstest", "utility to get joystick button mappings");
+	po::options_description opt3("Hidden options");
+	opt3.add_options()
+	  ("songdir", po::value<std::vector<std::string> >(&songdirs)->composing(), "");
 	// Process flagless options as songdirs
 	po::positional_options_description p;
 	p.add("songdir", -1);
 	po::options_description cmdline;
 	cmdline.add(opt1).add(opt2);
 	po::variables_map vm;
-	// Load more arguments from performous.conf files
+	// Load the arguments
 	try {
-		po::store(po::command_line_parser(argc, argv).options(cmdline).positional(p).run(), vm);
-		{
-			std::ifstream conf((getHomeDir() / ".config" / "performous.conf").string().c_str());
-			po::store(po::parse_config_file(conf, opt2), vm);
-		}
-		{
-			std::ifstream conf("/etc/performous.conf");
-			po::store(po::parse_config_file(conf, opt2), vm);
-		}
+		po::options_description allopts(cmdline);
+		allopts.add(opt3);
+		po::store(po::command_line_parser(argc, argv).options(allopts).positional(p).run(), vm);
 	} catch (std::exception& e) {
 		std::cout << cmdline << std::endl;
 		std::cout << "ERROR: " << e.what() << std::endl;
@@ -226,11 +288,11 @@ int main(int argc, char** argv) {
 	}
 	po::notify(vm);
 	if (vm.count("version")) {
-		std::cout << PACKAGE << ' ' << VERSION << std::endl;
+		// Already printed the version string in the beginning...
 		return 0;
 	}
 	if (vm.count("help")) {
-		std::cout << cmdline << std::endl;
+		std::cout << cmdline << "  any arguments without a switch are interpreted as song folders.\n" << std::endl;
 		return 0;
 	}
 	// Initialize audio for pdevhelp, michelp and the rest of the program
@@ -247,6 +309,7 @@ int main(int argc, char** argv) {
 		for (da::playback::devlist_t::const_iterator it = l.begin(); it != l.end(); ++it) {
 			std::cout << boost::format("  %1% %|10t|%2%\n") % it->name() % it->desc();
 		}
+		std::cout << std::flush;
 		return 0;
 	}
 	if (vm.count("michelp")) {
@@ -258,35 +321,34 @@ int main(int argc, char** argv) {
 		  "                                    Note: only the first four at most will be used\n"
 		  "  --mics channels=1,rate=44100      One mic; will try to get 44100 Hz if available\n\n"
 		  "Multiple --mics options may be specified and all the successfully opened inputs\n"
-		  "are be assigned as players until the maximum of four players are configured.\n" << std::endl;
+		  "are assigned as players until the maximum of four players are configured.\n" << std::endl;
 		std::cout << "Capture devices available:" << std::endl;
 		for (da::record::devlist_t::const_iterator it = l.begin(); it != l.end(); ++it) {
 			std::cout << boost::format("  %1% %|10t|%2%\n") % it->name() % it->desc();
 		}
+		std::cout << std::flush;
 		return 0;
 	}
 	// Read config files
 	try {
 		readConfig();
-		char const* env_data_dir = getenv("PERFORMOUS_DATA_DIR");
-		if(env_data_dir) config["system/path_data"].sl().insert(config["system/path_data"].sl().begin(),std::string(env_data_dir));
 	} catch (std::exception& e) {
 		std::cerr << e.what() << std::endl;
 		return EXIT_FAILURE;
 	}
 	// Override XML config for options that were specified from commandline or performous.conf
-	if (vm.count("width")) config["graphic/window_width"].i() = vm["width"].as<int>();
-	if (vm.count("height")) config["graphic/window_height"].i() = vm["height"].as<int>();
-	if (vm.count("fs-width")) config["graphic/fs_width"].i() = vm["fs-width"].as<int>();
-	if (vm.count("fs-height")) config["graphic/fs_height"].i() = vm["fs-height"].as<int>();
-	if (vm.count("theme")) config["game/theme"].s() = vm["theme"].as<std::string>();
-	if (vm.count("fs")) config["graphic/fullscreen"].b() = true;
-	if (vm.count("fps")) config["graphic/fps"].b() = true;
 	confOverride(songdirs, "system/path_songs");
 	confOverride(mics, "audio/capture");
 	confOverride(pdevs, "audio/playback");
+	getPaths(); // Initialize paths before other threads start
+	if (vm.count("jstest")) { // Joystick test program
+		std::cout << std::endl << "Joystick utility - Touch your joystick to see buttons here" << std::endl
+		<< "Hit ESC (window focused) to quit" << std::endl << std::endl;
+		jstestLoop();
+		return 0;
+	}
 	// Run the game init and main loop
-	mainLoop();
+	mainLoop(songlist);
 	return 0; // Do not remove. SDL_Main (which this function is called on some platforms) needs return statement.
 }
 
