@@ -1,13 +1,14 @@
 #include "opengl_text.hh"
+
 #include <boost/lexical_cast.hpp>
 #include <libxml++/libxml++.h>
 #include <pango/pangocairo.h>
-#include <math.h>
+#include <cmath>
 #include <iostream>
 #include <sstream>
 
 OpenGLText::OpenGLText(TThemeTxtOpenGL& _text, double m) {
-	if (_text.fontfamily.empty()) _text.fontfamily = "Sans";
+	if (_text.fontfamily.empty()) _text.fontfamily = "Arial";
 
 	PangoAlignment alignment = PANGO_ALIGN_LEFT;
 	if (_text.fontalign == "start") alignment = PANGO_ALIGN_LEFT;
@@ -195,13 +196,13 @@ void SvgTxtTheme::draw(std::vector<std::string> const& _text) {
 	draw(tmp);
 }
 
-void SvgTxtTheme::draw(std::string _text) {
+void SvgTxtTheme::draw(std::string _text, float alpha) {
 	std::vector<TZoomText> tmp;
 	TZoomText t;
 	t.string = _text;
 	t.factor = 1.0;
 	tmp.push_back(t);
-	draw(tmp);
+	draw(tmp, alpha);
 }
 
 void SvgTxtTheme::draw(std::vector<TZoomText> const& _text, float alpha) {
