@@ -1,7 +1,9 @@
 #include "screen_practice.hh"
 
 #include "util.hh"
-#include "configuration.hh"
+#include "fs.hh"
+#include "record.hh"
+#include "joystick.hh"
 
 ScreenPractice::ScreenPractice(std::string const& name, Audio& audio, Capture& capture):
   Screen(name), m_audio(audio), m_capture(capture)
@@ -16,37 +18,32 @@ void ScreenPractice::enter() {
 		m_vumeters.push_back(b = new ProgressBar(getThemePath("vumeter_bg.svg"), getThemePath("vumeter_fg.svg"), ProgressBar::VERTICAL, 0.136, 0.023));
 		b->dimensions.screenBottom().left(-0.4 + i * 0.2).fixedWidth(0.04);
 	}
+	unsigned int sr = m_audio.getSR();
+	m_samples.push_back(Sample(getPath("sounds/drum_bass.ogg"), sr));
+	m_samples.push_back(Sample(getPath("sounds/drum_snare.ogg"), sr));
+	m_samples.push_back(Sample(getPath("sounds/drum_hi-hat.ogg"), sr));
+	m_samples.push_back(Sample(getPath("sounds/drum_tom1.ogg"), sr));
+	m_samples.push_back(Sample(getPath("sounds/drum_cymbal.ogg"), sr));
+	//m_samples.push_back(Sample(getPath("sounds/drum_tom2.ogg"), sr));
 }
 
 void ScreenPractice::exit() {
 	m_vumeters.clear();
+	m_samples.clear();
 	theme.reset();
 }
 
 void ScreenPractice::manageEvent(SDL_Event event) {
 	ScreenManager * sm = ScreenManager::getSingletonPtr();
-	if (event.type == SDL_KEYDOWN) {
-		switch(event.key.keysym.sym) {
-			case SDLK_ESCAPE:
-			case SDLK_q:
-				sm->activateScreen("Intro");
-				break;
-			case SDLK_SPACE:
-			case SDLK_PAUSE:
-				m_audio.togglePause();
-				break;
-			default: // nothing to do, fixes warnings
-				break;
-		}
-	}
-	switch( event.type ) {
-		case SDL_JOYAXISMOTION:
-		case SDL_JOYHATMOTION:
-		case SDL_JOYBALLMOTION:
-		case SDL_JOYBUTTONDOWN:
-		case SDL_JOYBUTTONUP:
-			//check_joystick_event(event, m_audio);
-			break;
+	input::NavButton nav(input::getNav(event));
+	if (nav == input::CANCEL || nav == input::START || nav == input::SELECT) sm->activateScreen("Intro");
+	else if (nav == input::PAUSE) m_audio.togglePause();
+	else if (nav == input::VOLUME_DOWN) --config["audio/music_volume"];
+	else if (nav == input::VOLUME_UP) ++config["audio/music_volume"];
+	else if (event.type == SDL_JOYBUTTONDOWN // Play drum sounds here
+	  && input::Private::devices[event.jbutton.which].type_match(input::DRUMS)) {
+		int b = input::buttonFromSDL(input::Private::devices[event.jbutton.which].type(), event.jbutton.button);
+		if (b != -1) m_audio.play(m_samples[unsigned(b) % m_samples.size()], "audio/fail_volume");
 	}
 }
 
