@@ -1,21 +1,22 @@
 #include "songs.hh"
 
+#include "configuration.hh"
 #include "fs.hh"
-#include "screen.hh"
+#include "song.hh"
+#include "database.hh"
+#include "i18n.hh"
+
 #include <boost/bind.hpp>
 #include <boost/format.hpp>
 #include <boost/lexical_cast.hpp>
 #include <boost/regex.hpp>
 #include <libxml++/libxml++.h>
-// #include <tr1/random>
-#include <cstdlib>
 #include <algorithm>
-#include <fstream>
 #include <iostream>
-#include <iterator>
 #include <stdexcept>
+#include <cstdlib>
 
-Songs::Songs(std::string const& songlist): m_songlist(songlist), math_cover(), m_order(), m_dirty(false), m_loading(false), m_needShuffle(false) {
+Songs::Songs(Database & database, std::string const& songlist): m_songlist(songlist), math_cover(), m_database(database), m_order(), m_dirty(false), m_loading(false) {
 	reload();
 }
 
@@ -26,12 +27,7 @@ Songs::~Songs() {
 
 void Songs::reload() {
 	if (m_loading) return;
-	// Copy songdirs from config into m_songdirs
-	ConfigItem::StringList const& sd = config["system/path_songs"].sl();
-	m_songdirs.clear();
-	std::transform(sd.begin(), sd.end(), std::inserter(m_songdirs, m_songdirs.end()), pathMangle);
 	// Run loading thread
-	m_needShuffle = false;
 	m_loading = true;
 	m_thread.reset(new boost::thread(boost::bind(&Songs::reload_internal, boost::ref(*this))));
 }
@@ -42,25 +38,24 @@ void Songs::reload_internal() {
 		m_songs.clear();
 		m_dirty = true;
 	}
-	for (SongDirs::const_iterator it = m_songdirs.begin(); m_loading && it != m_songdirs.end(); ++it) {
-		if (!fs::is_directory(*it)) { std::cout << ">>> Not scanning: " << *it << " (no such directory)" << std::endl; continue; }
-		std::cout << ">>> Scanning " << *it << std::endl;
+	Paths paths = getPathsConfig("system/path_songs");
+	for (Paths::iterator it = paths.begin(); m_loading && it != paths.end(); ++it) {
+		if (!fs::is_directory(*it)) { m_debug << ">>> Not scanning: " << *it << " (no such directory)" << std::endl; continue; }
+		m_debug << ">>> Scanning " << *it << std::endl;
 		size_t count = m_songs.size();
 		reload_internal(*it);
 		size_t diff = m_songs.size() - count;
-		if (diff > 0 && m_loading) std::cout << diff << " songs loaded" << std::endl;
+		if (diff > 0 && m_loading) m_debug << diff << " songs loaded" << std::endl;
 	}
 	if (m_loading) dumpSongs_internal(); // Dump the songlist to file (if requested)
 	m_loading = false;
-	m_needShuffle = true;  // Force shuffle
 }
 
 void Songs::reload_internal(fs::path const& parent) {
-	static int randomIdx = 0;
 	namespace fs = fs;
-	if (std::distance(parent.begin(), parent.end()) > 20) { std::cout << ">>> Not scanning: " << parent.string() << " (maximum depth reached, possibly due to cyclic symlinks)" << std::endl; return; }
+	if (std::distance(parent.begin(), parent.end()) > 20) { m_debug << ">>> Not scanning: " << parent.string() << " (maximum depth reached, possibly due to cyclic symlinks)" << std::endl; return; }
 	try {
-		boost::regex expression(".*\\.[Tt][Xx][Tt]$");
+		boost::regex expression("(.*\\.txt|^song\\.ini|.*\\.sm)$", boost::regex_constants::icase);
 		boost::cmatch match;
 		for (fs::directory_iterator dirIt(parent), dirEnd; m_loading && dirIt != dirEnd; ++dirIt) {
 			fs::path p = dirIt->path();
@@ -68,24 +63,23 @@ void Songs::reload_internal(fs::path const& parent) {
 			std::string name = p.leaf(); // File basename (notes.txt)
 			std::string path = p.directory_string(); // Path without filename
 			path.erase(path.size() - name.size());
-			if (name.size() < 5 || !regex_match(name.substr(name.size() - 4).c_str(),match,expression)) continue;
+			if (!regex_match(name.c_str(), match, expression)) continue;
 			try {
-				Song* s = new Song(path, name);
-				s->randomIdx = ++randomIdx; // Not so random during loading, they are shuffled after load is finished
+				boost::shared_ptr<Song>s(new Song(path, name));
+				s->randomIdx = rand();
 				boost::mutex::scoped_lock l(m_mutex);
-				m_songs.push_back(boost::shared_ptr<Song>(s));
+				m_songs.push_back(s);
 				m_dirty = true;
 			} catch (SongParserException& e) {
 				if (e.silent()) continue;
-				std::ostringstream oss;
-				oss << "-!- Error in " << path << "\n    " << name;
-				if (e.line()) oss << " line " << e.line();
-				oss << ": " << e.what() << std::endl;
-				std::cerr << oss.str(); // More likely to be atomic when written as one string
+				// Construct error message
+				m_debug << "-!- Error in " << path << "\n    " << name;
+				if (e.line()) m_debug << " line " << e.line();
+				m_debug << ": " << e.what() << std::endl;
 			}
 		}
 	} catch (std::exception const& e) {
-		std::cout << "Error accessing " << parent << std::endl;
+		m_debug << "Error accessing " << parent << e.what() << std::endl;
 	}
 }
 
@@ -114,32 +108,10 @@ class Songs::RestoreSel {
 };
 
 void Songs::update() {
-	if (m_dirty) filter_internal();
-	// Shuffle the songlist if shuffle is finished and all songs are already filtered
-	if (m_needShuffle && !m_dirty) {
-		randomize_internal();
-		math_cover.setTarget(0, m_songs.size());
-		m_needShuffle = false;
-	}
-}
-
-void Songs::randomize() {
-	RestoreSel restore(*this);
-	randomize_internal();
-}
-
-void Songs::randomize_internal() {
-	/* TR1-based random number generation
-	namespace rnd = std::tr1;
-	rnd::random_device gendev;  // Random number generator (using /dev/urandom usually)
-	rnd::mt19937 gen(gendev);  // Make Mersenne Twister random number generator, seeded with random_device.
-	for (SongVector::const_iterator it = m_filtered.begin(); it != m_filtered.end(); ++it) (*it)->randomIdx = gen();
-	*/
-	std::srand(std::time(NULL));
-	// Assign the songs randomIdx that is used for sorting in the "random" mode
-	for (SongVector::const_iterator it = m_filtered.begin(); it != m_filtered.end(); ++it) (*it)->randomIdx = std::rand();
-	m_order = 0;  // Use randomIdx sort mode
-	sort_internal();
+	if (m_dirty) filter_internal(); // Update with newly loaded songs
+	// A hack to move to the first song when the song screen is entered the first time
+	static bool first = true;
+	if (first) { first = false; math_cover.setTarget(0, 0); math_cover.setTarget(0, size()); }
 }
 
 void Songs::setFilter(std::string const& val) {
@@ -150,12 +122,17 @@ void Songs::setFilter(std::string const& val) {
 
 void Songs::filter_internal() {
 	boost::mutex::scoped_lock l(m_mutex);
+	// Print messages when loading has finished
+	if (!m_loading) {
+		std::cerr << m_debug.str();
+		m_debug.str(""); m_debug.clear();
+	}
 	m_dirty = false;
 	RestoreSel restore(*this);
 	try {
 		SongVector filtered;
 		for (SongVector::const_iterator it = m_songs.begin(); it != m_songs.end(); ++it) {
-			if (regex_search(it->get()->strFull(), boost::regex(m_filter ,boost::regex_constants::icase))) filtered.push_back(*it);
+			if (regex_search(it->get()->strFull(), boost::regex(m_filter, boost::regex_constants::icase))) filtered.push_back(*it);
 		}
 		m_filtered.swap(filtered);
 	} catch (...) {
@@ -167,21 +144,21 @@ void Songs::filter_internal() {
 
 namespace {
 
-    /// A functor that compares songs based on a selected member field of them.
-    template<typename Field> class CmpByField {
-	    Field Song::* m_field;
-      public:
-	    /** @param field a pointer to the field to use (pointer to member) **/
-	    CmpByField(Field Song::* field): m_field(field) {}
-	    /// Compare left < right
-	    bool operator()(Song const& left , Song const& right) {
+	/// A functor that compares songs based on a selected member field of them.
+	template<typename Field> class CmpByField {
+		Field Song::* m_field;
+	  public:
+		/** @param field a pointer to the field to use (pointer to member) **/
+		CmpByField(Field Song::* field): m_field(field) {}
+		/// Compare left < right
+		bool operator()(Song const& left , Song const& right) {
 		    return left.*m_field < right.*m_field;
-	    }
-	    /// Compare *left < *right
-	    bool operator()(boost::shared_ptr<Song> const& left, boost::shared_ptr<Song> const& right) {
-		    return operator()(*left, *right);
-	    }
-    };
+		}
+		/// Compare *left < *right
+		bool operator()(boost::shared_ptr<Song> const& left, boost::shared_ptr<Song> const& right) {
+			return operator()(*left, *right);
+		}
+	};
 
     /// A helper for easily constructing CmpByField objects
     template <typename T> CmpByField<T> comparator(T Song::*field) { return CmpByField<T>(field); }
@@ -194,26 +171,27 @@ namespace {
 		return path.substr(pos, path.size() - pos - 1);
 	}
 
-
-    static char const* order[] = {
-	    "random order",
-	    "by song",
-	    "by artist",
-	    "by edition",
-	    "by genre",
-	    "by path"
-    };
-
-    static const int orders = sizeof(order) / sizeof(*order);
+	static const int orders = 7;
 
 }
 
 std::string Songs::sortDesc() const {
-	std::string str = order[m_order];
+	std::string str = "";
+	switch (m_order) {
+	  case 0: str = _("random order"); break;
+	  case 1: str = _("sorted by song"); break;
+	  case 2: str = _("sorted by artist"); break;
+	  case 3: str = _("sorted by edition"); break;
+	  case 4: str = _("sorted by genre"); break;
+	  case 5: str = _("sorted by path"); break;
+	  case 6: str = _("sorted by language"); break;
+	  default: throw std::logic_error("Internal error: unknown sort order in Songs::sortDesc");
+	}
 	if (!empty()) {
 		if (m_order == 3) str += " (" + current().edition + ")";
 		if (m_order == 4) str += " (" + current().genre + ")";
 		if (m_order == 5) str += " (" + pathtrim(current().path) + ")";
+		if (m_order == 6) str += " (" + current().language + ")";
 	}
 	return str;
 }
@@ -225,10 +203,6 @@ void Songs::sortChange(int diff) {
 	sort_internal();
 }
 
-namespace {
-	int rnd(int n) { return rand() % n; }
-}
-
 void Songs::sort_internal() {
 	switch (m_order) {
 	  case 0: std::stable_sort(m_filtered.begin(), m_filtered.end(), comparator(&Song::randomIdx)); break;
@@ -237,6 +211,7 @@ void Songs::sort_internal() {
 	  case 3: std::sort(m_filtered.begin(), m_filtered.end(), comparator(&Song::edition)); break;
 	  case 4: std::sort(m_filtered.begin(), m_filtered.end(), comparator(&Song::genre)); break;
 	  case 5: std::sort(m_filtered.begin(), m_filtered.end(), comparator(&Song::path)); break;
+	  case 6: std::sort(m_filtered.begin(), m_filtered.end(), comparator(&Song::language)); break;
 	  default: throw std::logic_error("Internal error: unknown sort order in Songs::sortChange");
 	}
 }
