@@ -3,13 +3,16 @@
 #include "fs.hh"
 #include "configuration.hh"
 #include "video_driver.hh"
-#include <Magick++.h>
-#include <boost/bind.hpp>
+#include "image.hh"
+
 #include <fstream>
 #include <stdexcept>
 #include <sstream>
 #include <vector>
 
+#include <cctype>
+
+#include <boost/cstdint.hpp>
 using boost::uint32_t;
 
 float Dimensions::screenY() const {
@@ -21,102 +24,26 @@ float Dimensions::screenY() const {
 	throw std::logic_error("Dimensions::screenY(): unknown m_screenAnchor value");
 }
 
-namespace {
-	bool isPow2(unsigned int val) {
-		unsigned int count = 0;
-		do { if (val & 1) ++count; } while (val >>= 1);
-		return val == 1;
-	}
-
-	unsigned int nextPow2(unsigned int val) {
-		unsigned int ret = 1;
-		while (ret < val) ret *= 2;
-		return ret;
-	}
-
-	unsigned int prevPow2(unsigned int val) {
-		unsigned int ret = 1;
-		while ((ret*2) < val) ret *= 2;
-		return ret;
-	}
-
-	bool diffRGBA(uint32_t a, uint32_t b, unsigned threshold = 5) {
-		for (unsigned byte = 0; byte < 4; ++byte) {
-			int aByte = (a >> (byte * 8)) & 0xFF;
-			int bByte = (b >> (byte * 8)) & 0xFF;
-			if (unsigned(std::abs(aByte - bByte)) > threshold) return true;
-		}
-		return false;
-	}
-}
-
 bool checkExtension(std::string const& extension) {
 	std::istringstream iss(reinterpret_cast<char const*>(glGetString(GL_EXTENSIONS)));
 	for (std::string ext; iss >> ext;) if (ext == extension) return true;
 	return false;
 }
 
-template <typename T> void loader(T& target, std::string filename, bool autocrop) {
-	std::string themefilename = getThemePath(filename);
-	if (std::ifstream(themefilename.c_str()).is_open()) filename = themefilename;
-	else if (!std::ifstream(filename.c_str()).is_open()) throw std::runtime_error("File not found: " + filename);
-	if (filename.size() > 4 && filename.substr(filename.size() - 4) == ".svg") {
-		rsvg_init();
-		GError* pError = NULL;
-		// Find SVG dimensions (in pixels)
-		RsvgHandle* svgHandle = rsvg_handle_new_from_file(filename.c_str(), &pError);
-		if (pError) {
-			g_error_free(pError);
-			throw std::runtime_error("Unable to load " + filename);
-		}
-		RsvgDimensionData svgDimension;
-		rsvg_handle_get_dimensions (svgHandle, &svgDimension);
-		rsvg_handle_free(svgHandle);
-		double factor = config["graphic/svg_lod"].f();
-		unsigned int w = nextPow2(svgDimension.width*factor);
-		unsigned int h = nextPow2(svgDimension.height*factor);
-		// Load and raster the SVG
-		GdkPixbuf* pb = rsvg_pixbuf_from_file_at_size(filename.c_str(), w, h, &pError);
-		if (pError) {
-			g_error_free(pError);
-			throw std::runtime_error("Unable to load " + filename);
-		}
-		target.load(w, h, pix::CHAR_RGBA, gdk_pixbuf_get_pixels(pb), float(svgDimension.width)/svgDimension.height);
-		gdk_pixbuf_unref(pb);
-		rsvg_term();
-	} else {
-		Magick::Image image;
-		Magick::Blob blob;
-		try {
-			image.read(filename);
-			image.magick("RGBA");
-			image.write(&blob);
-			unsigned w = image.columns();
-			unsigned ybegin = 0, yend = image.rows();
-			uint32_t const* buf = static_cast<uint32_t const*>(blob.data());
-			if (autocrop) {
-				uint32_t bordercol = buf[0];
-				for (bool mismatch = false; ybegin < yend; ++ybegin) {
-					for (unsigned x = 0; x < w; ++x) if (diffRGBA(buf[ybegin * w + x], bordercol)) { mismatch = true; break; }
-					if (mismatch) break;
-				}
-				for (bool mismatch = false; yend > ybegin; --yend) {
-					unsigned y = yend - 1;
-					for (unsigned x = 0; x < w; ++x) if (diffRGBA(buf[y * w + x], bordercol)) { mismatch = true; break; }
-					if (mismatch) break;
-				}
-			}
-			target.load(w, yend - ybegin, pix::CHAR_RGBA, reinterpret_cast<unsigned char const*>(buf + ybegin * w));
-		}
-		catch (Magick::Exception&) // add error handling
-		{
-			throw std::runtime_error("Image Error");
-		} 
-	}
+template <typename T> void loader(T& target, fs::path name) {
+	std::string const filename = name.string();
+	if (!fs::exists(name)) throw std::runtime_error("File not found: " + filename);
+	// Get file extension in lower case
+	std::string ext = name.extension();
+	std::for_each(ext.begin(), ext.end(), static_cast<int(*)(int)>(std::tolower));
+	
+	if (ext == ".svg") loadSVG(target, filename);
+	else if (ext == ".png") loadPNG(target, filename);
+	else loadJPEG(target, filename);
 }
 
-Texture::Texture(std::string const& filename) { loader(*this, filename, false); }
-Surface::Surface(std::string const& filename, bool autocrop) { loader(*this, filename, autocrop); }
+Texture::Texture(std::string const& filename) { loader(*this, filename); }
+Surface::Surface(std::string const& filename) { loader(*this, filename); }
 
 // Stuff for converting pix::Format into OpenGL enum values
 namespace {
@@ -152,18 +79,21 @@ void Texture::load(unsigned int width, unsigned int height, pix::Format format,
 	glTexParameterf(type(), GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST);
 	// When texture area is large, bilinear filter the original
 	glTexParameterf(type(), GL_TEXTURE_MAG_FILTER, GL_LINEAR);
-	glTexParameterf(type(), GL_TEXTURE_MAX_LEVEL, 0);
 	// The texture wraps over at the edges (repeat)
 	glTexParameterf(type(), GL_TEXTURE_WRAP_S, GL_REPEAT);
 	glTexParameterf(type(), GL_TEXTURE_WRAP_T, GL_REPEAT);
+	//glTexParameterf(type(), GL_TEXTURE_MAX_LEVEL, 1);
+	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, 16.0f);
+	glTexParameteri(type(), GL_GENERATE_MIPMAP, GL_TRUE);
 	PixFmt const& f = getPixFmt(format);
 	glPixelStorei(GL_UNPACK_SWAP_BYTES, f.swap);
 	// Load the data into texture
-	if (checkExtension("GL_ARB_texture_non_power_of_two")) { // Use OpenGL 2.0 functionality 
+	if ((isPow2(width) && isPow2(height)) || 
+	  checkExtension("GL_ARB_texture_non_power_of_two")) { // Use OpenGL 2.0 functionality 
 		glTexImage2D(type(), 0, GL_RGBA, width, height, 0, f.format, f.type, buffer);	
 	} else {
-		// TODO: Test for OpenGL extension GL_ARB_texture_non_power_of_two and if not found, 
-		// use gluScaleImage to upscale the texture to nextPow2 dimensions before calling 
+		// TODO: Test for OpenGL extension GL_ARB_texture_non_power_of_two and if not found,
+		// use gluScaleImage to upscale the texture to nextPow2 dimensions before calling
 		// glTexImage2D (if it isn't pow2 already).
 		// speeds it up... trust me you need it for now at least! :P
 		// TODO: remove when cairo is fixed.
@@ -179,7 +109,7 @@ void Texture::load(unsigned int width, unsigned int height, pix::Format format,
 		// BIG FAT WARNING: Do not even think of using ARB_texture_rectangle here!
 		// Every developer of the game so far has tried doing so, but it just cannot work.
 		// (1) no repeat => cannot texture
-		// (2) coordinates not normalized => would require special hackery elsewhere		
+		// (2) coordinates not normalized => would require special hackery elsewhere
 		// Just don't do it in Surface class, thanks. -Tronic
 		glTexImage2D(type(), 0, GL_RGBA, newWidth, newHeight, 0, f.format, f.type, &outBuf[0]);
 	}
@@ -194,7 +124,7 @@ void Surface::load(unsigned int width, unsigned int height, pix::Format format,
 	UseTexture texture(m_texture);
 	PixFmt const& f = getPixFmt(format);
 	glPixelStorei(GL_UNPACK_SWAP_BYTES, f.swap);
-	glTexImage2D(m_texture.type(), 0, GL_RGBA, width, height, 0, f.format, f.type, buffer);	
+	glTexImage2D(m_texture.type(), 0, GL_RGBA, width, height, 0, f.format, f.type, buffer);
 }
 
 void Surface::draw() const {
