@@ -2,14 +2,13 @@
 
 #include <glibmm/ustring.h>
 #include <glibmm/convert.h>
+#include <sstream>
 
 void convertToUTF8( std::stringstream &_stream, std::string _filename ) {
 	try {
 		Glib::convert(_stream.str(), "UTF-8", "UTF-8"); // Test if input is UTF-8
 	} catch(...) {
-		if( _filename.empty() ) _filename = std::string("stream");
-
-		std::cerr << "WARNING: " << _filename << " is not UTF-8.\n  Assuming CP1252 for now. Use recode CP1252..UTF-8 */*.txt to convert your files." << std::endl;
+		if (!_filename.empty()) std::cerr << "WARNING: " << _filename << " is not UTF-8.\n  Assuming CP1252 for now. Use recode CP1252..UTF-8 */*.txt to convert your files." << std::endl;
 		try {
 			_stream.str(Glib::convert(_stream.str(), "UTF-8", "CP1252")); // Convert from Microsoft CP1252
 		} catch (...) {
