@@ -2,17 +2,23 @@
 
 #include "config.hh"
 #include "glutil.hh"
-#include "screen.hh"
+#include "fs.hh"
+#include "image.hh"
 #include "util.hh"
+#include "joystick.hh"
+#include <boost/date_time.hpp>
+#include <fstream>
 #include <SDL.h>
 
-unsigned s_width;
-unsigned s_height;
+namespace {
+	unsigned s_width;
+	unsigned s_height;
+}
 
 unsigned int screenW() { return s_width; }
 unsigned int screenH() { return s_height; }
 
-Window::Window(unsigned int width, unsigned int height, int fs, unsigned int fs_width, unsigned int fs_height): m_windowW(width), m_windowH(height), m_fsW(fs_width), m_fsH(fs_height) {
+Window::Window(unsigned int width, unsigned int height, bool fs): m_windowW(width), m_windowH(height), m_fullscreen(fs) {
 	std::atexit(SDL_Quit);
 	if( SDL_Init(SDL_INIT_VIDEO|SDL_INIT_JOYSTICK) ==  -1 ) throw std::runtime_error("SDL_Init failed");
 	SDL_WM_SetCaption(PACKAGE " " VERSION, PACKAGE);
@@ -21,28 +27,19 @@ Window::Window(unsigned int width, unsigned int height, int fs, unsigned int fs_
 		SDL_WM_SetIcon(icon, NULL);
 		SDL_FreeSurface(icon);
 	}
-	m_fullscreen = (fs != 0);
+	// SDL_SetVideoMode not called yet => get the desktop resolution for fs mode
+	SDL_VideoInfo const* vi = SDL_GetVideoInfo();
+	m_fsW = vi->current_w;
+	m_fsH = vi->current_h;
 	resize();
 	SDL_ShowCursor(SDL_DISABLE);
 	SDL_EnableUNICODE(SDL_ENABLE);
-	SDL_EnableKeyRepeat(80, 80);
+	input::SDL::init();
 }
 
+Window::~Window() { }
+
 void Window::blank() {
-	glClearColor (0.0f, 0.0f, 0.0f, 1.0f);
-	glDisable (GL_DEPTH_TEST);
-	glDisable(GL_CULL_FACE);
-	glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-	glEnable (GL_BLEND);
-	s_width = screen->w;
-	s_height = screen->h;
-	if (s_height < 0.56f * s_width) s_width = round(s_height / 0.56f);
-	if (s_height > 0.8f * s_width) s_height = round(0.8f * s_width);
-	glViewport(0.5f * (screen->w - s_width), 0.5f * (screen->h - s_height), s_width, s_height);
-	glMatrixMode(GL_PROJECTION);
-	glLoadIdentity();
-	float h = virtH();
-	glOrtho(-0.5f, 0.5f, 0.5f * h, -0.5f * h, -1.0f, 1.0f);
 	glClear(GL_COLOR_BUFFER_BIT);
 }
 
@@ -59,6 +56,18 @@ bool Window::getFullscreen() {
 	return m_fullscreen;
 }
 
+void Window::screenshot() {
+	Image img;
+	img.w = m_fullscreen ? m_fsW : m_windowW;
+	img.h = m_fullscreen ? m_fsH : m_windowH;
+	img.data.resize(((img.w + 3) & ~3) * img.h * 3);
+	glReadPixels(0, 0, img.w, img.h, GL_RGB, GL_UNSIGNED_BYTE, &img.data[0]);
+	fs::path filename = getHomeDir() / ("Performous_" + to_iso_string(boost::posix_time::second_clock::local_time()) + ".png");
+	writePNG(filename.string(), img);
+	std::cout << ">>> Screenshot taken: " << filename << " (" << img.w << "x" << img.h << ")" << std::endl;
+}
+
+
 void Window::resize() {
 	unsigned width = m_fullscreen ? m_fsW : m_windowW;
 	unsigned height = m_fullscreen ? m_fsH : m_windowH;
@@ -76,5 +85,37 @@ void Window::resize() {
 	SDL_GL_SetAttribute(SDL_GL_ACCUM_ALPHA_SIZE, 0);
 	screen = SDL_SetVideoMode(width, height, 0, SDL_OPENGL | SDL_RESIZABLE | (m_fullscreen ? SDL_FULLSCREEN : 0));
 	if (!screen) throw std::runtime_error(std::string("SDL_SetVideoMode failed: ") + SDL_GetError());
+
+	s_width = screen->w;
+	s_height = screen->h;
+	if (!m_fullscreen) {
+		config["graphic/window_width"].i() = s_width;
+		config["graphic/window_height"].i() = s_height;
+	}
+	if (s_height < 0.56f * s_width) s_width = round(s_height / 0.56f);
+	if (s_height > 0.8f * s_width) s_height = round(0.8f * s_width);
+	glViewport(0.5f * (screen->w - s_width), 0.5f * (screen->h - s_height), s_width, s_height);
+	// Set flags
+	glClearColor (0.0f, 0.0f, 0.0f, 1.0f);
+	glDisable(GL_DEPTH_TEST);
+	glDisable(GL_CULL_FACE);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
+	glShadeModel(GL_SMOOTH);
+	glEnable(GL_BLEND);
+	// Set projection
+	glMatrixMode(GL_PROJECTION);
+	glLoadIdentity();
+	float h = virtH();
+	// stump: under MSVC, near and far are #defined to nothing for compatibility with ancient code, hence the underscores.
+	const float near_ = 1.5f; // This determines FOV: the value is your distance from the monitor (the unit being the width of the Performous window)
+	const float far_ = 100.0f; // How far away can things be seen
+	// Set model-view matrix
+	glMatrixMode(GL_MODELVIEW);
+	glLoadIdentity();
+	const float f = 0.9f; // Avoid texture surface being exactly at the near plane (MacOSX fix)
+	glFrustum(-0.5f * f, 0.5f * f, 0.5f * h * f, -0.5f * h * f, f * near_, far_);
+	glTranslatef(0.0f, 0.0f, -near_);  // So that z = 0.0f is still on monitor surface
+
 }
 
