@@ -14,9 +14,12 @@
 #include <libxml++/libxml++.h>
 
 #include <boost/filesystem.hpp>
+#include <boost/lexical_cast.hpp>
+#include <boost/program_options.hpp>
 
 #include "pak.h"
 #include "ipuconv.hh"
+#include "ss_cover.hh"
 
 namespace fs = boost::filesystem;
 
@@ -27,7 +30,8 @@ int sleepts = -1;
 xmlpp::Node::PrefixNsMap nsmap;
 std::string ns;
 const bool video = true;
-const bool mkvcompress = false;
+const bool mkvcompress = true;
+const bool oggcompress = true;
 
 // LibXML2 logging facility
 extern "C" void xmlLogger(void* logger, char const* msg, ...) { if (logger) *(std::ostream*)logger << msg; }
@@ -85,7 +89,7 @@ Filename_ safename(Glib::ustring const& str) {
 		if (*it >= 0x7F && *it < 0xA0) continue; // Additional control characters
 		if (first && *it == '.') continue;
 		if (first && *it == '-') continue;
-		if (*it == '&') { ret += " et "; continue; }
+		if (*it == '&') { ret += " and "; continue; }
 		if (*it == '%') { ret += " percent "; continue; }
 		if (*it == '$') { ret += " dollar "; continue; }
 		if (forbidden.find(*it) != Glib::ustring::npos) { ret += "_"; continue; }
@@ -127,6 +131,7 @@ void parseSentence(xmlpp::Node* node) {
 #include "adpcm.h"
 
 unsigned getLE16(char* buf) { unsigned char* b = reinterpret_cast<unsigned char*>(buf); return b[0] | (b[1] << 8); }
+unsigned getLE32(char* buf) { unsigned char* b = reinterpret_cast<unsigned char*>(buf); return b[0] | (b[1] << 8) | (b[2] << 16) | (b[3] << 24); }
 
 struct Song {
 	std::string dataPakName, title, artist, genre, edition, year;
@@ -185,9 +190,13 @@ void music(Song& song, PakFile const& dataFile, PakFile const& headerFile, fs::p
 			if (l2 != 0 || r2 != 0) karaoke = true;
 		}
 	}
-	std::string ext = ".wav";
-	writeMusic(song.music = outPath / ("music" + ext), pcm[0], sr);
-	if (karaoke) writeMusic(song.vocals = outPath / ("vocals" + ext), pcm[1], sr);
+	std::string ext;
+	writeMusic(song.music = outPath / ("music.wav"), pcm[0], sr);
+	if (karaoke) writeMusic(song.vocals = outPath / ("vocals.wav"), pcm[1], sr);
+	if( oggcompress ) {
+		song.music = outPath / ("music.ogg");
+		if (karaoke) song.vocals = outPath / ("vocals.ogg");
+	}
 }
 
 struct Match {
@@ -210,6 +219,42 @@ std::string xmlFix(std::vector<char> const& data) {
 
 std::string filename(boost::filesystem::path const& p) { return *--p.end(); }
 
+void saveTxtFile(xmlpp::NodeSet &sentence, const fs::path &path, const Song &song, const std::string singer = "") {
+	fs::path file_path;
+
+	if( singer.empty() ) {
+		file_path = path / "notes.txt";
+	} else {
+		file_path = path;
+		file_path /= safename(std::string("notes") + " (" + singer + ")" + ".txt");
+	}
+	txtfile.open(file_path.string().c_str());
+	if( singer.empty() )
+		txtfile << "#TITLE:" << song.title << std::endl;
+	else
+		txtfile << "#TITLE:" << song.title << " (" << singer << ")" << std::endl;
+	txtfile << "#ARTIST:" << song.artist << std::endl;
+	if (!song.genre.empty()) txtfile << "#GENRE:" << song.genre << std::endl;
+	if (!song.year.empty()) txtfile << "#YEAR:" << song.year << std::endl;
+	if (!song.edition.empty()) txtfile << "#EDITION:" << song.edition << std::endl;
+	//txtfile << "#LANGUAGE:English" << std::endl; // Detect instead of hardcoding? 
+	if (!song.music.empty()) txtfile << "#MP3:" << filename(song.music) << std::endl;
+	if (!song.vocals.empty()) txtfile << "#VOCALS:" << filename(song.vocals) << std::endl;
+	if (video && mkvcompress) {
+		txtfile << "#VIDEO:video.m4v" << std::endl;
+	} else {
+		txtfile << "#VIDEO:video.mpg" << std::endl;
+	}
+	txtfile << "#COVER:cover.jpg" << std::endl;
+	//txtfile << "#BACKGROUND:background.jpg" << std::endl;
+	txtfile << "#BPM:" << song.tempo << std::endl;
+	ts = 0;
+	sleepts = -1;
+	std::for_each(sentence.begin(), sentence.end(), parseSentence);
+	txtfile << 'E' << std::endl;
+	txtfile.close();
+}
+
 struct Process {
 	Pak const& pak;
 	Process(Pak const& p): pak(p) {}
@@ -255,35 +300,54 @@ struct Process {
 			dom.get_document()->write_to_file((path / "notes.xml").string(), "UTF-8");
 			Pak dataPak(song.dataPakName);
 			std::cerr << ">>> Extracting and decoding music" << std::endl;
-			music(song, dataPak[id + "/music.mib"], pak["export/" + id + "/music.mih"], path);
-			txtfile.open((path / "notes.txt").string().c_str());
-			txtfile << "#TITLE:" << song.title << std::endl;
-			txtfile << "#ARTIST:" << song.artist << std::endl;
-			if (!song.genre.empty()) txtfile << "#GENRE:" << song.genre << std::endl;
-			if (!song.year.empty()) txtfile << "#YEAR:" << song.year << std::endl;
-			if (!song.edition.empty()) txtfile << "#EDITION:" << song.edition << std::endl;
-			//txtfile << "#LANGUAGE:English" << std::endl; // Detect instead of hardcoding? 
-			if (!song.music.empty()) txtfile << "#MP3:" << filename(song.music) << std::endl;
-			if (!song.vocals.empty()) txtfile << "#VOCALS:" << filename(song.vocals) << std::endl;
-			if (video && mkvcompress) {
-				txtfile << "#VIDEO:video.m4v" << std::endl;
+			try {
+				music(song, dataPak[id + "/music.mib"], pak["export/" + id + "/music.mih"], path);
+			} catch (...) {
+				std::cerr << "  >>> European DVD failed, trying American (WIP)" << std::endl;
+				// Tracks on my example
+				// 0 => video (ipu)
+				// 1 and 2 => adpcm song (left/right)
+				// 3 and 4 => adpcm vocals (left/right)
+				std::vector<char> offsets_header;
+				std::vector<char> offsets_data;
+				dataPak[id + "/mus+vid.ind"].get(offsets_header, 0, 0x68);
+				unsigned int track_count = getLE32(&offsets_header[4]);
+				dataPak[id + "/mus+vid.ind"].get(offsets_data, 0x68);
+				unsigned int offset = 0;
+				/*
+				for( unsigned int i = 0 ; i < offsets_data.size() ; i+=2) {
+					std::vector<char> chunk_data;
+					unsigned int size = getLE16(&offsets_data[i]) << 4;
+					dataPak[id + "/mus+vid.iav"].get(chunk_data, offset, size);
+					offset+=size;
+				}
+				*/
+			}
+			std::cerr << ">>> Extracting lyrics" << std::endl;
+			xmlpp::NodeSet sentences = dom.get_document()->get_root_node()->find("/" + ns + "MELODY/" + ns + "SENTENCE", nsmap);
+			if(!sentences.empty()) {
+				// Sentences not inside tracks (normal songs)
+				std::cerr << "  >>> Solo track" << std::endl;
+				saveTxtFile(sentences, path, song);
 			} else {
-				txtfile << "#VIDEO:video.mpg" << std::endl;
+				xmlpp::NodeSet tracks = dom.get_document()->get_root_node()->find("/" + ns + "MELODY/" + ns + "TRACK", nsmap);
+				if( !tracks.empty()) {
+					for( xmlpp::NodeSet::iterator it = tracks.begin() ; it != tracks.end() ; ++it ) {
+						xmlpp::Element& elem = dynamic_cast<xmlpp::Element&>(**it);
+						std::string singer = elem.get_attribute("Artist")->get_value();
+						std::cerr << "  >>> Track from " << singer << std::endl;
+						sentences = dom.get_document()->get_root_node()->find((*it)->get_path() + "/" + ns + "SENTENCE", nsmap);
+						saveTxtFile(sentences, path, song, singer);
+					}
+				} else {
+					throw std::runtime_error("Unable to find any sentences in melody XML");
+				}
 			}
-			txtfile << "#COVER:cover.jpg" << std::endl;
-			//txtfile << "#BACKGROUND:background.jpg" << std::endl;
-			txtfile << "#BPM:" << song.tempo << std::endl;
-			ts = 0;
-			sleepts = -1;
-			xmlpp::NodeSet n = dom.get_document()->get_root_node()->find("/" + ns + "MELODY/" + ns + "SENTENCE", nsmap); // Sentences not inside tracks (normal songs)
-			if (n.empty()) n = dom.get_document()->get_root_node()->find("/" + ns + "MELODY/" + ns + "TRACK[1]/" + ns + "SENTENCE", nsmap); // Try to get the first track (duet songs?)
-			if (n.empty()) throw std::runtime_error("Unable to find any sentences in melody XML");
-			std::for_each(n.begin(), n.end(), parseSentence);
-			txtfile << 'E' << std::endl;
-			txtfile.close();
 			std::cerr << ">>> Extracting cover image" << std::endl;
-			// FIXME: use internally instead of separate program
-			std::system(("ss_cover_conv \"" + dvdPath + "/pack_ee.pak\" " + id + " \"" + path.string() + "/cover.jpg\"").c_str());
+			try {
+				SingstarCover c = SingstarCover(dvdPath + "/pack_ee.pak", boost::lexical_cast<unsigned int>(id));
+				c.write(path.string() + "/cover.jpg");
+			} catch (...) {}
 			remove = "";
 			if (video) {
 				std::cerr << ">>> Extracting video" << std::endl;
@@ -292,9 +356,22 @@ struct Process {
 				std::cerr << ">>> Converting video" << std::endl;
 				IPUConv(ipudata, (path / "video.mpg").string());
 				// FIXME: use some library (preferrably ffmpeg):
+				if (oggcompress) {
+					std::cerr << ">>> Compressing audio into music.ogg/vocals.ogg" << std::endl;
+					std::string cmd = "oggenc \"" + (path / "music.wav").string() + "\"";
+					std::cerr << cmd << std::endl;
+					if (std::system(cmd.c_str()) == 0) { // FIXME: std::system return value is not portable
+						fs::remove(path / "music.wav");
+					}
+					cmd = "oggenc \"" + (path / "vocals.wav").string() + "\"";
+					std::cerr << cmd << std::endl;
+					if (std::system(cmd.c_str()) == 0) { // FIXME: std::system return value is not portable
+						fs::remove(path / "vocals.wav");
+					}
+				}
 				if (mkvcompress) {
 					std::cerr << ">>> Compressing video and audio into music.mkv" << std::endl;
-					std::string cmd = "ffmpeg -i \"" + (path / "video.mpg").string() + "\" -vcodec libx264 -vpre hq -crf 25 -threads 0 -album \"" + song.edition + "\" -author \"" + song.artist + "\" -comment \"" + song.genre + "\" -title \"" + song.title + "\" \"" + (path / "video.m4v\"").string();
+					std::string cmd = "ffmpeg -i \"" + (path / "video.mpg").string() + "\" -vcodec libx264 -vpre hq -crf 25 -threads 0 -metadata album=\"" + song.edition + "\" -metadata author=\"" + song.artist + "\" -metadata comment=\"" + song.genre + "\" -metadata title=\"" + song.title + "\" \"" + (path / "video.m4v\"").string();
 					std::cerr << cmd << std::endl;
 					if (std::system(cmd.c_str()) == 0) { // FIXME: std::system return value is not portable
 						fs::remove(path / "video.mpg");
@@ -433,11 +510,35 @@ struct FindSongs {
 };
 
 int main( int argc, char **argv) {
-	if (argc < 2 || argc > 3) {
-		std::cerr << "Usage: " << argv[0] << " dvdPath [track id or name]" << std::endl;
-		return EXIT_FAILURE;
+	std::string video, audio, song;
+	namespace po = boost::program_options;
+	po::options_description opt("Options");
+	opt.add_options()
+	  ("help,h", "you are viewing it")
+	  ("dvd", po::value<std::string>(&dvdPath), "path to Singstar DVD root")
+	  ("list,l", "list tracks only")
+	  ("song", po::value<std::string>(&song), "only extract the given track (ID or partial name)")
+	  ("video", po::value<std::string>(&video)->default_value("mkv"), "specify video format (none, mkv, mpeg2)")
+	  ("audio", po::value<std::string>(&audio)->default_value("ogg"), "specify audio format (none, ogg, wav)")
+	  ;
+	// Process the first flagless option as dvd, the second as song
+	po::positional_options_description pos;
+	pos.add("dvd", 1);
+	pos.add("song", 1);
+	po::options_description cmdline;
+	cmdline.add(opt);
+	po::variables_map vm;
+	// Load the arguments
+	try {
+		po::store(po::command_line_parser(argc, argv).options(opt).positional(pos).run(), vm);
+		po::notify(vm);
+		if (dvdPath.empty()) throw std::runtime_error("No Singstar DVD path specified. Enter a path to a folder with pack_ee.pak in it.");
+		// TODO: process audio and video options and throw if they have incorrect values
+	} catch (std::exception& e) {
+		std::cout << cmdline << std::endl;
+		std::cout << "ERROR: " << e.what() << std::endl;
+		return 1;
 	}
-	dvdPath = argv[1];
 	std::string pack_ee = dvdPath + "/pack_ee.pak"; // Note: lower case (ISO-9660)
 	if (!fs::exists(pack_ee)) {
 		if (fs::exists(dvdPath + "/Pack_EE.PAK")) { // Note: capitalization (UDF)
@@ -445,14 +546,17 @@ int main( int argc, char **argv) {
 			  "Singstar DVDs have UDF and ISO-9660 filesystems on them. Your disc is mounted\n"
 			  "as UDF and this causes some garbled data files, making ripping it impossible.\n\n"
 			  "Please remount the disc as ISO-9660 and try again. E.g. on Linux:\n"
-			  "# mount -t iso9600 /dev/cdrom " << dvdPath << std::endl;
+			  "# mount -t iso9660 /dev/cdrom " << dvdPath << std::endl;
 		} else std::cerr << "No Singstar DVD found. Enter a path to a folder with pack_ee.pak in it." << std::endl;
 		return EXIT_FAILURE;
 	}
 	nsmap["ss"] = "http://www.singstargame.com";
 	Pak p(pack_ee);
-	FindSongs f = std::for_each(p.files().begin(), p.files().end(), FindSongs(argc > 2 ? argv[2] : ""));
+	FindSongs f = std::for_each(p.files().begin(), p.files().end(), FindSongs(song));
 	std::cerr << f.songs.size() << " songs found" << std::endl;
-	std::for_each(f.songs.begin(), f.songs.end(), Process(p));
+	if (vm.count("list")) {
+		// TODO: dump list of songs
+	}
+	else std::for_each(f.songs.begin(), f.songs.end(), Process(p));
 }
 
